library in_avsbeat;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Beat Input Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2011
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $41010000}

uses
  Windows,
  common in '..\common.pas';

{$R *.RES}

type
  TDetails = class(TObject)
  public
    beat: Boolean;
    bpm: TLevel;
  end;

var
  msg: UINT = 0;
  bAtom: ATOM;
  window: HWND;
  mutexModules: THandle = INVALID_HANDLE_VALUE;
  modules: array of PLightningModule;

function windowProc(hwnd: HWND;	uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  i: Integer;
begin
  Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
  if uMsg = msg then
  begin
    if WaitForSingleObject(mutexModules, 1000) = WAIT_OBJECT_0 then
    begin
      for i := 0 to Length(modules) - 1 do
      begin
        with TDetails(modules[i].details) do
        begin
          beat := True;
          if lParam > levelMax then
            bpm := levelMax
          else
            bpm := lParam;
        end;
      end;
      ReleaseMutex(mutexModules);
    end;
  end;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  mutexModules := CreateMutex(nil, False, nil);
  SetLength(modules, 0);
  Result := return('IBeat|Audio', name, nameLength);
end;

procedure finish(); cdecl;
begin
  if mutexModules <> INVALID_HANDLE_VALUE then
    CloseHandle(mutexModules);
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  bWndClass: TWndClassA;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'Beat';
    settings      := False;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := 0;
    inputTooltip  := '';
    outputs       := 2;
    outputTooltip := 'Beat Pulse|BPM Value';
    details       := TDetails.Create();
    reserved      := nil;
  end;
  if WaitForSingleObject(mutexModules, 1000) = WAIT_OBJECT_0 then
  begin
    SetLength(modules, Length(modules) + 1);
    modules[Length(modules) - 1] := Result;
    ReleaseMutex(mutexModules);
  end;
  if msg = 0 then
  begin
    msg := RegisterWindowMessage('lightning_beat');
    bWndClass.hInstance := hInstance;
    with bWndClass do
    begin
      style         := 0;
      lpfnWndProc   := @windowProc;
      cbClsExtra    := 0;
      cbWndExtra    := 0;
      hIcon         := 0;
      hCursor       := 0;
      hbrBackground := 0;
      lpszMenuName  := nil;
      lpszClassName := 'TLightningBeat';
    end;
    bATOM  := RegisterClass(bWndClass);
    window := CreateWindow('TLightningBeat', 'Lightning Beat', 0, 0, 0, 0, 0, 0, 0, hInstance, nil);
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
var
  i, j: Integer;
begin
  if WaitForSingleObject(mutexModules, 1000) = WAIT_OBJECT_0 then
  begin
    for i := 0 to Length(modules) - 1 do
      if modules[i] = lightningModule then
      begin
        for j := i to Length(modules) - 2 do
          modules[j] := modules[j + 1];
        Break;
      end;
    if Length(modules) > 0 then
      SetLength(modules, Length(modules) - 1);
    if (Length(modules) = 0) and (msg <> 0) then
    begin
      DestroyWindow(window);
      UnregisterClass('TLightningBeat', hInstance);
      msg := 0;
    end;
    ReleaseMutex(mutexModules);
  end;
  TDetails(lightningModule^.details).Free();
  Dispose(lightningModule);
end;

function action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
begin
  Result := True;
  details := TDetails(lightningModule^.details);
  if output = 1 then
  begin
    if details.beat then
    begin
      details.beat := False;
      createSpectrum(bridge, levelMax);
    end
    else
      createSpectrum(bridge, levelMin);
  end
  else // else if output = 2 then
    createSpectrum(bridge, details.bpm);
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  //
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
begin
  //
end;

function save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return('', settings, settingsLength);
end;

exports
  init, finish, create, destroy, action, show, hide, open, save;

end.
